import numpy as np
import matplotlib, json, codecs
import matplotlib.pyplot as plt
import scipy, scipy.constants, scipy.stats
import json
import mjsutils as m

constants = json.loads( codecs.open( './constants.json','r','utf-8').read() )

def loadConstant(*args):
    r = []
    for name in args:
        r.append(constants[name]['value'])
    return [*r]

h = scipy.constants.h
N0, Nuclear_magneton, kB, R, N0 = loadConstant('Avogadro Constant','Nuclear Magneton','Boltzmann Constant','Gas Constant','Avogadro Constant')
elements = json.loads( codecs.open( './elements.json','r','utf-8').read() )

def Cn_fast(B,T,material,T_epsilon = 1e-6):
    "Return the heat capacity per mole equations 9.11 "
    #this 'fast' version does the differentialtion explisitylly.
    #this makes it vectorisable in B and T. woop woop.
    g = elements[material]['Nuclear G Factor']['value']
    J = elements[material]['Nuclear Spin']['value']
    #make small extra T around T for the gradient and the curviture
    elepsilon = Nuclear_magneton * g * np.sqrt( B**2 + elements[material]['Intrinsic Magnetic Field']['value']**2 ) 
    T_a = T-2.0*T_epsilon
    T_b = T-T_epsilon
    T_c = T
    T_d = T+T_epsilon
    T_e = T+2.0*T_epsilon
    Z_a = T*0
    Z_c = T*0
    Z_e = T*0
    for m in np.arange(-1*J,J+1,step=1,dtype=np.float64):
        epsilon = elepsilon * m  
        Z_a = Z_a +  np.exp( -1*epsilon / (kB * T_a))
        Z_c = Z_c +  np.exp( -1*epsilon / (kB * T_c))
        Z_e = Z_e +  np.exp( -1*epsilon / (kB * T_e))
    S_a = R *  ( T_c*np.log(Z_c) - T_a*np.log(Z_a) ) / ( T_c - T_a )
    S_c = R *  ( T_e*np.log(Z_e) - T_c*np.log(Z_c) ) / ( T_e - T_c )
    Cb = T *  ( S_c - S_a) / (T_d - T_b)
    return Cb

def Cn_Cu(B,T,T_epsilon = 1e-6):
    "Return the heat capacity per mole equations 9.11 "
    #this 'fast' version does the differentialtion explisitylly.
    #this makes it vectorisable in B and T. woop woop.
    g = 1.52118952 # g factor
    J = 1.5 # spin
    bi = 0.04
    elepsilon = Nuclear_magneton * g * np.sqrt( B**2 + bi**2 ) 
    T_a = T-2.0*T_epsilon
    T_b = T-T_epsilon
    T_c = T
    T_d = T+T_epsilon
    T_e = T+2.0*T_epsilon
    Z_a = T*0
    Z_c = T*0
    Z_e = T*0
    for m in np.arange(-1*J,J+1,step=1,dtype=np.float64):
        epsilon = elepsilon * m  
        Z_a = Z_a +  np.exp( -1*epsilon / (kB * T_a))
        Z_c = Z_c +  np.exp( -1*epsilon / (kB * T_c))
        Z_e = Z_e +  np.exp( -1*epsilon / (kB * T_e))
    S_a = R *  ( T_c*np.log(Z_c) - T_a*np.log(Z_a) ) / ( T_c - T_a )
    S_c = R *  ( T_e*np.log(Z_e) - T_c*np.log(Z_c) ) / ( T_e - T_c )
    Cb = T *  ( S_c - S_a) / (T_d - T_b)
    return Cb

def Cn_In(B,T,T_epsilon = 1e-6):
    "Return the heat capacity per mole equations 9.11 "
    #this 'fast' version does the differentialtion explisitylly.
    #this makes it vectorisable in B and T. woop woop.
    g = 1.228 # g factor
    J = 4.5 # spin
    e2qQ = -50.0e6 * h # scale back by h to get e^2qQ
    cos2theta = 0.5  # cos^2 (theta) approx 0.5 to the averave of all directions.
    T_a = T-2.0*T_epsilon
    T_b = T-T_epsilon
    T_c = T
    T_d = T+T_epsilon
    T_e = T+2.0*T_epsilon
    Z_a = T*0
    Z_c = T*0
    Z_e = T*0
    for m in np.arange(-1*J,J+1,step=1,dtype=np.float64):
        epsilon = elepsilon = Nuclear_magneton * g * B * m + e2qQ / (4.0*J*(2.0*J - 1.0)) * (3.0* cos2theta -1)/ 2.0 * ( 3*m**2 - J * (J+1)  )
        Z_a = Z_a +  np.exp( -1*epsilon / (kB * T_a))
        Z_c = Z_c +  np.exp( -1*epsilon / (kB * T_c))
        Z_e = Z_e +  np.exp( -1*epsilon / (kB * T_e))
    S_a = R *  ( T_c*np.log(Z_c) - T_a*np.log(Z_a) ) / ( T_c - T_a )
    S_c = R *  ( T_e*np.log(Z_e) - T_c*np.log(Z_c) ) / ( T_e - T_c )
    Cb = T *  ( S_c - S_a) / (T_d - T_b)
    return Cb

def Cn_shottky(B,T,material):
    'equ 10.4, page 219 F.Pobell Matter and Methdos....'
    g = elements[material]['Nuclear G Factor']['value']
    J = elements[material]['Nuclear Spin']['value']
    b = np.sqrt(elements[material]['Intrinsic Magnetic Field']['value']**2 + B**2)
    lambda_n = N0 * J *(J+1) * Nuclear_magneton**2 * g**2 / 3 / kB
    Cn = lambda_n * b**2 / T**2
    return Cn
